;
; TYPE_R: A remote TYPE for DOORS users.
;
; Courtesy of: The BBS of Excellence (312)398-2872
;
; To make this program work under COM2: change DataReg to 2F8h
;
cseg    segment  byte public 'CODE'
        assume   cs:cseg,ds:cseg,es:cseg
        org      100h
TYPE_R  proc     near
        jmp      MainCode   ; Skip over constant
;
Handle   dw      ?          ; Save area for file "handle"
ASCIIZ   dw      ?          ; Addr of ASCIIZ
DataReg  dw      03F8h      ; for COM1: 2F8h = COM2:
LineReg  dw      03FDh      ; for COM1: 2FDh = COM2:
ErrorMsg db      'Unable to open ',0
DataArea db      257 dup(0)
BadForm  db      13,7,10,'Format MUST be "TYPE_R drive:/subdir/filename"',13,10,0
;
MainCode:
        Call     ValidateEntry  ; Validate the Command line entry format
        Call     OpenFile       ; Go Open the file
        Call     ClearLine      ; Clear the data from COMx:
        Call     DisplayFile    ; Display the file
        Call     CloseFile      ; Close the file
        Int      20h            ; Terminate program
TYPE_R  endp

ClearLine Proc Near
           Mov     DX,DataReg      ; Point to the data
           IN      AL,DX           ; Read a byte
           IN      AL,DX           ; And the next one
ClearLineLoop:
           Mov     DX,LineReg      ; Get the Line Register address xFDh
           In      AL,DX           ; Get the Line status
           And     AL,1Fh          ; Isolate DR/OR/PE/FE/or BI active
           OR      AL,AL           ; Are they there?
           Jz      ExitReset       ; No...Exit to caller
           Mov     DX,DataReg      ; Point to the data register
           In      AL,DX           ; Reset data area
           Jmp     ClearLineLoop   ; Keep clearing any data
ExitReset:
ClearLine Endp

ValidateEntry  Proc    Near
           mov     SI,80h          ; point to the command buffer
           Xor     CX,CX           ; Init the loop counter
           Mov     CL,[SI]         ; Get the input length
           Cmp     CL,0            ; Was there no input ?
           JZ      BadEntry        ; True...then abort
SkipLoop:
           Inc     SI              ; Point to the next byte
           Mov     DL,[SI]         ; Get the character
           Cmp     DL,' '          ; Is it Blank ?
           Jnz     ByteFound       ; No...continue
           Loop    SkipLoop        ; Else, keep on looking
ByteFound:
           Mov     ASCIIZ,SI       ; Save the pointer
           Cmp     DL,97           ; Is it in lower case ?
           Jb      UpperCase       ; No..don't change it
           Sub     DL,32           ; Make it upper case
UpperCase:
           Cmp     DL,'A'          ; Is it Alpha ?
           Jb      BadEntry        ; No...Abort
           Cmp     DL,'E'          ; Is it too high ?
           Ja      BadEntry        ; Yes..Abort
           Inc     SI              ; Point to the colon
           Mov     DL,[SI]         ; Get the character
           Cmp     DL,':'          ; Is it a Colon ?
           Jnz     BadEntry        ; No..Abort
           Inc     SI              ; Point to the Slash
           Mov     DL,[SI]         ; Get the character
           Cmp     DL,'\'          ; Is it a slash ?
           Jnz     BadEntry        ; No...Abort
;
LoopToEnd:
           Inc     SI              ; Skip to the next character
           Mov     DL,[SI]         ; Get the character
           Cmp     DL,13           ; End of the line ?
           Jnz     LoopToEnd       ; No...keep on scanning
           Xor     AL,AL           ; Init the work area
           Mov     [SI],AL         ; Add an end-of-ASCIIZ indicator
           Jmp     ExitValidation  ; Exit this routine
BadEntry:
          Lea      SI,BadForm      ; Point to the error message
          Call     SendMsg         ; Send it
          Int      20h        ; Abort processing
ExitValidation:
           Ret
ValidateEntry    Endp

OpenFile  Proc   Near
          Mov    DX,ASCIIZ  ; Point to the ASCIIZ string
          Xor    AL,AL      ; No file protection
          Mov    AH,61      ; Set up the command
          Int    21h        ; Do the Open
          Jc     CantOpen   ; If unable to open, tell the user
          Mov    Handle,AX  ; Save the "handle"
          Jmp    ExitOpen   ; Exit this routine
CantOpen:
          Lea    SI,ErrorMsg  ; Point to the error message
          Call   SendMsg    ; Display the message
          Int    20h        ; Terminate processing
ExitOpen:
          Ret               ; Return to the caller
OpenFile  Endp

CloseFile Proc   Near
          Mov    DX,Handle  ; Get the file handle
          Xor    AX,AX      ; Init the work register
          Mov    AH,62      ; Set up the command
          Int    21h        ; Execute it
          Ret               ; Return to caller
CloseFile Endp

DisplayFile Proc Near
DisplayLoop:
          Call   ReadFile   ; Read some data
          Or     AX,AX      ; End of file ?
          Jz     ExitDisplay ; Yes...terminate
          Lea    SI,DataArea ; Point to the data
          Mov    CX,AX       ; Get the Length
;
LineLp:
          Mov    AL,[SI]     ; Get the character
          Call   SendByte    ; Display it
          Inc    SI          ; Point to the next character
          Loop   LineLp      ; Do it for the full length
          Jmp    DisplayLoop ; Keep on displaying
ExitDisplay:
          Ret               ; Return to caller
DisplayFile Endp

ReadFile  Proc   Near
          Mov    BX,Handle  ; Get the file handle name
          Lea    DX,DataArea  ; Point to the file area
          Mov    CX,256     ; Read 256 bytes
          Xor    AX,AX      ; Init the work register
          Mov    AH,63      ; Set up the command
          Int    21h        ; Execute it
          Ret
ReadFile  Endp

SendMsg   Proc   Near
LineLoop:
          Mov    AL,[SI]    ; Get the Character
          Cmp    AL,0       ; End of the Line ?
          Jz     ExitLine   ; Yes...exit
          Call   SendByte   ; Else display the character
          Inc    SI         ; Point to the next character
          Jmp    LineLoop   ; And continue
ExitLine:
          Ret
SendMsg   Endp

SendByte  Proc   Near
          Cmp    AL,1Ah          ; EOF character ?
          Jz     ExitByte        ; Yes..don't display it
          Push   AX              ; Save the byte
WaitLineLoop:
          Mov     DX,LineReg      ; Get the Line Register address xFDh
          In      AL,DX           ; Get the Line status
          And     AL,20h          ; Is the Xmit buffer empty ?
          Cmp     AL,20h          ; Is it?
          Jne     WaitLineLoop    ; No...Exit to caller
          Pop    AX              ; Restore the byte
          Mov    DX,DataReg      ; Point to output port
          Out    DX,AL           ; Output the character
          Mov    DL,AL           ; Get the character
          Mov    AH,2            ; Setup to output to video
          Int    21h             ; Do it....
ExitByte:
          Ret
SendByte  Endp
CSEG      Ends
          End     TYPE_R
